/**
  *   ANTLR grammar for COFOL.  
  *   Follows TPTP formats mostly for wffs.
  *   EVAL: $ prefix signifies evaluation functor
  *   14 April 2011
  *   Type names can be any name, no special requirements.
  *   BE SURE TO SET CORRECT GENERATOR PATH!  e.g., /colog2.02/sam/fol/antlr
  *   
  */
grammar Fol ;
//options { backtrack = true ; } // braces or parens for wffs ;=) 
                                 // comment out when testing with '{'..'}' 
@header {
package sam.fol.antlr ;
import sam.lang.* ; 
import sam.fol.* ; 
import java.util.Vector ; 
}
@lexer::header { package sam.fol.antlr ; }

@members {
/** Store Theory */
Fol fol ;
}

test 	:  {fol = new Fol();} theory ; 

theory 	:    (f=formula   {fol.theory.add($f.formula);} '.')* ;

formula returns [Formula formula]	:	
         w=wff   
        {$formula = new Formula($w.value) ;  }
         ;

wff returns [WFF value] :
      p=predicate {$value = new WFF(WFF.FUNCTOR); $value.operands.add($p.value) ;}
    | '~' w=wff {$value = new WFF(WFF.NOT); $value.operands.add($w.value) ;}
    | '!' '[' v=varlist ']' ':' w=wff 
        {$value = new WFF(WFF.ALL); $value.operands.add($v.value) ; $value.operands.add($w.value) ;}
    | '?' '[' v=varlist ']' ':' w=wff 
        {$value = new WFF(WFF.EXISTS); $value.operands.add($v.value) ; $value.operands.add($w.value) ;}
    |  '{'  w1=wff                 // use ('{' | ')') with options { backtrack = true ; } 
       (  '&' {$value=new WFF(WFF.AND) ; $value.operands.add($w1.value) ; } 
              w2=wff {$value.operands.add($w2.value) ; } 
                ('&' w2=wff {$value.operands.add($w2.value) ; })* 
        |  '|' {$value=new WFF(WFF.OR) ; $value.operands.add($w1.value) ; } 
              w2=wff {$value.operands.add($w2.value) ; } 
              ('|' w2=wff {$value.operands.add($w2.value) ; })*
        |  '=>' {$value=new WFF(WFF.IF) ; $value.operands.add($w1.value) ; } 
              w2=wff {$value.operands.add($w2.value) ; } 
        |  '<=>' {$value=new WFF(WFF.IFF) ; $value.operands.add($w1.value) ; }
              w2=wff {$value.operands.add($w2.value) ; } 
       )
              '}'                // use ('{' | ')') with options { backtrack = true ; } 
    ;   
    
predicate returns [Functor value]:	
         f=functor {$value = $f.value ; } // tricky
      	   (PRED t=term  { Vector<Term> v1 = new Vector<Term>() ;  v1.add($value) ; v1.add($t.value) ;    $value = new Functor($PRED.text,v1) ; }   )*  // 0 or 1
	 | v=variable PRED t=term  {Vector<Term> v2 = new Vector<Term>() ; v2.add($variable.value) ; v2.add($term.value) ; $value = new Functor($PRED.text,v2) ; }
	 | n=number PRED t=term {Vector<Term> v3 = new Vector<Term>() ; v3.add($n.value) ; v3.add($t.value) ; $value = new Functor($PRED.text,v3) ; }  // N.B. {..} brackets needed
	 | o=operator PRED t=term {Vector<Term> v3 = new Vector<Term>() ; v3.add($o.value) ; v3.add($t.value) ; $value = new Functor($PRED.text,v3) ; }  // N.B. {..} brackets needed
	;

functor  returns [Functor value]  :  
    name {$value = new Functor($name.value,0) ; } 
   |  name '(' terms ')' {$value = new Functor($name.value, $terms.value) ; }           
    ; 

operator returns [Functor value] :
   | UOP t=term {Vector<Term> v = new Vector<Term>() ;  v.add($t.value) ; $value = new Functor($UOP.text,v) ; } 
   | '('t1=term bop t2=term ')' {Vector<Term> v = new Vector<Term>() ;  v.add($t1.value) ; v.add($t2.value) ;$value = new Functor($BOP.text,v) ; } 
   ;
   
terms returns [Vector<Term> value] :  
   {$value = new Vector<Term>() ;}  t=term {$value.add($t.value);}  (',' t=term {$value.add($t.value);} )* ; 

term returns [Term value] : 
   variable {$value = $variable.value ; }
   | number  {$value = $number.value ;} 
   | functor {$value = $functor.value ; }   
   | operator {$value = $operator.value ; }; 

number returns [Functor value]:	
   NUMERIC+  {$value = new Functor($number.text,0) ; } 
   ;
	
varlist returns [Vector<TypedVariable> value]  :	
   {$value = new Vector<TypedVariable>() ;} v=typedvariable {$value.add($v.value);} (',' v=typedvariable {$value.add($v.value);})* ; 

variable returns [Variable value] : 
   (UPPER|'_') (LOWER | UPPER | '_' | NUMERIC)*  // like Prolog
   {$value= new Variable($variable.text) ; } ; 

typedvariable returns [TypedVariable value] :
    variable { $value = new TypedVariable($variable.text,"dom") ; } 
    | v= variable ':' n=name 
           { $value = new TypedVariable($v.text,$n.value) ; } 
    ;

name returns [String value]:	
   EVAL* LOWER (LOWER | UPPER | '_' | NUMERIC)*  {$value = $name.text ;}  
   | QUOTED { $value = $name.text ; } 
   | DQUOTED { $value = $name.text ; } 
   ; 
   
PRED 	:	 EVAL* ('=' | '!=' | '<' |'<=' | '>' | '>=' | '==' | '!==')   ;  // predicate symbols

LOWER	:	'a'..'z' ;

UPPER 	:	'A'..'Z' ; 

NUMERIC :	 '0'..'9' ; 

bop :  
    EVAL* ('#' | '*' | '+' | '/' | '- ' | '^' | '&')   ; // N.B. space after '- ' for subtraction

UOP  :    EVAL*  '-'     ; // N.B. no space after negative  
    
EVAL 	:	'$' ;

WS : (' '|'\t'|'\n'|'\r')+ {skip();} ;

SL_COMMENT : ('%'|'//') ( options {greedy=false;} : . )* ('\n'|'\r') {skip();} ;

ML_COMMENT : '/*' ( options {greedy=false;} : . )* '*/' {skip();} ; 

QUOTED 	:	 '\'' ( options {greedy=false;} : .)* '\'' ;    

DQUOTED :	 '\"' ( options {greedy=false;} : .)* '\"' ;    

/*
This version dispenses with wff annotations.
A theory is simply a finite sequence of wffs.
All wffs are considered as axioms for the theory.
To propose a goal, use an axiom like this:

      (p(a,b) => goal).

To propose a refutation, use either of the following axioms:

      (p(a,b) => false).   

or 

      ~p(a,b).
      
This is a notational simplification, compared to 
previous versions.

*/

