package sam.fol ; 

import java.util.* ; 
import java.io.* ;
import java.net.* ;
import sam.lang.* ; 
import sam.fol.antlr.* ;

/**
  *  Fol, First-order logic.
  *  The class participates with the Fol.g ANTLR 
  *  parser to produce a Fol theory.
  */
public class Fol  { 

   public Vector<Formula> theory ;      // wffs+warnings

   public Fol() { 
      this.theory = new Vector<Formula>() ; 
   } 

   /**
     *  Display first-order theory,
     *  shows warnings associated with 
     *  reading wffs in Formulas.
     */
   public String toString() {
      StringBuffer sb = new StringBuffer() ; 
      Iterator it = this.theory.iterator() ; 
      while (it.hasNext())   { 
        sb.append(it.next().toString() ) ;
        sb.append(".\n") ; 
      }
      return sb.toString() ;   
   } 

   /**
     *  Convert to colog theory using method
     */
   public Theory colog(String method) { 
      return WFF.colog(this,method) ; // static method of WFF 
   }

   /**
     *  Translate FOL theory to colog theory, terse/verbose+fore/aft
     */
   public static Theory fol2colog(String urlstring,boolean terse,boolean fore) {
      String method = (terse ? "terse" : "verbose") ;
      method += (fore ? "+fore" : "+aft") ;
      Theory cologtheory = null ;  
      try { 
         URL url = new URL(urlstring) ; 
         InputStream in = url.openStream() ; 
         FolLoader loader = new FolLoader(in) ;  
         cologtheory = loader.fol.colog(method) ; 
         in.close() ; 
      }
      catch (Exception x) { System.out.println(x) ; }
      return cologtheory ;
   }
   
/*
   The fol2colog code is a little oblique: The static method foltocolog opens 
   a fol file and reads the wffs onto a Fol list and then converts that 
   list of wffs using a staic method of WFF, return the resulting list
   of colog rules.
*/ 

   public static void main(String[] args) { 
      String urlstring = "file:/colog14H/fol/list.fol" ;

      try { 
         URL url = new URL(urlstring) ; 
         InputStream in = url.openStream() ; 
         FolLoader loader = new FolLoader(in) ;  
         System.out.println(loader.fol) ; 
         in.close() ; 
         //loader.wait(1000) ; 
      } catch (Exception x) { System.out.println(x) ; }
      System.out.println("---------------------------") ; 

      boolean terse = true ,
              fore  = true ;
      System.out.println( Fol.fol2colog(urlstring,terse,fore) );
   }


} // end Fol class

