package sam.gui ; 

import javax.swing.* ; 
import java.awt.* ; 
import java.awt.event.* ; 
import java.io.* ; 
import java.net.* ; 
import java.awt.datatransfer.* ; 
import java.util.* ; 
import sam.fol.* ; 
import sam.fol.antlr.* ; 
import sam.lang.* ; 

/**
  *  FOLTool formulated as JPanel with action
  *  Opens fol and convert to colog. 
   */
public class FOLPane extends JPanel implements ActionListener{ 
   private boolean terse, fore ;
   private Fol fol ; 

   protected Colog colog ;  // parent gui component 
   protected JPanel toolpanel ; 
   protected JLabel foltoollabel ; 
   protected JButton open_button ; 
   protected JToggleButton terse_button ; 
   protected JToggleButton fore_button ; 
   protected JButton tocolog_button ;     // translates FOL
   protected JButton show_button ;   // saves translation
   protected JEditorPane folpane ;  // displays FOL
   protected String theoryDescription = "" ;

   public FOLPane(Colog colog) { 
      String os = System.getProperty("os.name") ; 
      this.terse = true ; this.fore = true ;
      this.colog = colog ;
      toolpanel = new JPanel() ;
      toolpanel.setLayout(new FlowLayout()) ; 
      foltoollabel = new JLabel("<html><font color=\"RED\">&hearts; </font> FOL TOOL") ;
      open_button = new JButton("open") ; 
      open_button.addActionListener(this) ;
      open_button.setActionCommand("open") ;
      open_button.createToolTip() ; 
      open_button.setToolTipText("open fol file with dialog") ; 
      terse_button = new JToggleButton("terse",true) ; 
      terse_button.createToolTip() ; 
      terse_button.setToolTipText("Use terse translation") ; 
      terse_button.addActionListener(this) ;
      terse_button.setActionCommand("terse") ;
      fore_button = new JToggleButton("fore",true) ; 
      fore_button.createToolTip() ;  
      fore_button.setToolTipText("Use fore translation") ;
      fore_button.addActionListener(this) ;
      fore_button.setActionCommand("fore") ;
      tocolog_button = new JButton("2colog") ;
      tocolog_button.setToolTipText("translate to colog") ;
      tocolog_button.addActionListener(this) ;
      tocolog_button.setActionCommand("2colog") ;
      show_button = new JButton("show") ;
      show_button.setToolTipText("show colog in aux pane") ;
      show_button.addActionListener(this) ;
      show_button.setActionCommand("show") ;
      //toolpanel.add(foltoollabel) ; 
      toolpanel.add(open_button) ; 
      toolpanel.add(terse_button) ; 
      toolpanel.add(fore_button) ; 
      toolpanel.add(tocolog_button) ; 
      toolpanel.add(show_button) ; 
      folpane = new JEditorPane() ; 
      //folpane.setToolTipText("<html><font size=\"+0\"><b>FOL area</b><br> &nbsp;Drop fol file here &nbsp;&nbsp;   <br>&nbsp;or use <b>open fol</b> button. <br> &nbsp;Parse error reports appear in console. &nbsp;&nbsp;     ") ;
      folpane.setBackground(new Color(245,255,245)) ; 
      folpane.setTransferHandler(new DropHandler()) ; 
      folpane.setFont(new Font((os.indexOf("Mac") > -1 ? "monospaced":"monospaced"),Font.PLAIN,14) ) ; 
      folpane.setBorder(BorderFactory.createTitledBorder("fol theory"));
      JScrollPane scroller = new JScrollPane(folpane) ; 

      this.setLayout(new BorderLayout()) ;
      this.add("South",toolpanel) ;
      this.add("Center",scroller) ;
   }

   /** 
     *  Clear the fol pane when new theory 
     *  opened in Colog GUI.
      */
   public void clear() {
      this.folpane.setText("") ;
      this.fol = null ;
   }

   public void actionPerformed(ActionEvent e) { 
      String command = e.getActionCommand() ; 
      // System.out.println(command) ;       
      if (command.equals("open")) {
         this.open() ; 
      }
      else if (command.equals("fore")) {
         if (fore_button.isSelected()) {
            fore_button.setToolTipText("Use fore translation") ; 
            fore_button.setText("fore") ; 
            this.fore = true ; 
         }
         else {
            fore_button.setToolTipText("Use aft translation") ; 
            fore_button.setText("aft") ; 
            this.fore = false ; 
         }
      }
      else if (command.equals("terse")) {
         if (terse_button.isSelected()) {
            terse_button.setToolTipText("Do terse translation") ; 
            terse_button.setText("terse") ; 
            this.terse = true ; 
         }
         else {
            terse_button.setToolTipText("Do verbose translation") ; 
            terse_button.setText("verbose") ; 
            this.terse = false ; 
         }
      }
      else if (command.equals("2colog")) {  
         try {
            String method = (this.terse ? "terse" : "verbose") ;
            method += (this.fore ? "+fore" : "+aft") ;
            this.colog.setTheory(fol.colog(method)) ;
         }
         catch (Exception exc) { 
            JOptionPane.showMessageDialog(this,exc.toString(),"fol-to-colog error",JOptionPane.INFORMATION_MESSAGE) ;
         }
      }   
      else if (command.equals("show")) {  
         Theory colog_theory = null ;
         boolean ok_to_show = false ;
         try {
            String method = (this.terse ? "terse" : "verbose") ;
            method += (this.fore ? "+fore" : "+aft") ;
            colog_theory = fol.colog(method) ; 
            ok_to_show = true ;
         }
         catch (Exception exc) { 
            JOptionPane.showMessageDialog(this,exc.toString(),"fol-to-colog error",JOptionPane.INFORMATION_MESSAGE) ;
         } 
         if (ok_to_show) 
            if (this.colog == null) System.out.println(colog_theory) ;
            else this.colog.auxArea.setText(colog_theory.toString()) ;
      }   

   }

   /**
     *  Open fol theory via file dialog.
     */
   protected synchronized void open() {
      JFileChooser fileChooser = new JFileChooser() ;  
      /////
      // attempt to start dialog in previous/last directory..
      String fs = null ; 
      if (theoryDescription.startsWith("file:"+File.separator+File.separator+"localhost")) 
         fs = theoryDescription.substring(16) ;
      else if (theoryDescription.startsWith("file:")) 
         fs = theoryDescription.substring(5) ; 
      if (fs != null) fileChooser.setSelectedFile(new File(fs)) ;   
      /////
      int result = fileChooser.showOpenDialog(this) ; 
      if (result == JFileChooser.APPROVE_OPTION) {    
         this.open(fileChooser.getSelectedFile().toURI().toString()) ; 
      }      
   } // end open()

   /**
     * open fol theory linked to url
     */
   public void open(String U) {  // can be called via command-line
      colog.clear() ;
      colog.theoryListModel.clear() ;
      colog.setTitle(colog.BANNER+" "+U) ;
      colog.theoryDescription = "" ;
      URL url = null ;
      try { 
         url = new URL(U) ; 
      }
      catch (Exception xxx) { 
         //System.out.println(xxx) ;  
         JOptionPane.showMessageDialog(this,xxx) ; 
      }
      if (url == null) return ; // do nothin if problem
      // get url
      this.fol = null ;   // null return if exception 
      boolean trouble = false ;
      try { 
         InputStream in = url.openStream() ; 
         FolLoader loader = new FolLoader(in) ;     
         this.fol = loader.fol ;
         this.folpane.setText(this.fol.toString()) ; 
         in.close() ;
      }
      catch (Exception x) { 
         trouble = true ; 
      }
      if (trouble) { 
         JOptionPane.showMessageDialog(this,
             "<html><b>Load errors") ; 
         //return ; // leave current theory alone
      }
      this.theoryDescription = url.toString() ;
   }

   /**
     * Extract the String flavor, which gives a reasonable
     * URL description for Java.  Need cross-platform flavor. 
     * Perhaps add DataFlavor.stringFlavor attempt to importData?
     * inner
      */
   class DropHandler extends TransferHandler { 

       public boolean canImport(JComponent comp, DataFlavor[] flavor) {
          return true ;  // 
       }

       public boolean importData(JComponent comp, Transferable t) { 
         DataFlavor[] flavors = t.getTransferDataFlavors() ; 
         int p = -1 ; 
         String s = "java.awt.datatransfer.DataFlavor[mimetype=text/uri-list;representationclass=java.lang.String]" ;
         for(int i = 0 ; i < flavors.length ; i++) {
            //System.out.println(flavors[i].toString()) ; // inspect the flavors
            if (flavors[i].toString().startsWith(s)) p = i ;
         }
         if (p != -1)  {   // use this String representation  Mac or Linux
            try { 
               String u = ((String)t.getTransferData(flavors[p])).trim() ; 
               //System.out.println(u) ; 
               open(u) ; 
               return true ;  
            } catch(Exception x1) { System.out.println(x1) ; }
         }  // end if
         else if (t.isDataFlavorSupported(DataFlavor.javaFileListFlavor))  {
            try { 
               java.util.List list = (java.util.List)t.getTransferData(DataFlavor.javaFileListFlavor);
               Iterator it = list.iterator() ; 
               //System.out.println(DataFlavor.javaFileListFlavor.toString()) ; 
               File file = (File)it.next() ;  // it is a FILE
               open(file.toURI().toURL().toString()) ;  
               return true ; 
            } catch (Exception x2) { System.out.println(x2) ; }
         }  // end else if 
         // otherwise ???
         return false ;
      }

   } // end inner DropHandler


   public static void main(String[] args) {
      JFrame frame = new JFrame("FOL TOOL") ;
      frame.setSize(500,700) ; 
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE) ;
      FOLPane folpane = new FOLPane(null) ;
      frame.add(folpane) ;
      frame.setVisible(true) ; 
   }
}

/*

Eventually foltool might be JavaEditor-based,
including fol wizard,  and
   compile means translate to colog
   run means use colog prover to run colog code

*/
