package sam.gui ; 

import sam.infer.* ; 
import javax.swing.* ; 
import java.util.* ; 
import java.awt.*; 
import java.awt.event.*; 
import java.awt.geom.* ;


/**
  * TreePane displays the model View for a sam.infer.Tree  
  * @version 4 August 2012
  * @author &copy John Fisher
  */
public class TreePane extends JPanel 
                            implements ActionListener { 

   public int rule ;       // -1 or else red box rule inference nodes  
   public int infcount ;   // how many nodes infer rule 
   public JLabel info ;       // where to show information, constructed in gui.Colog
   public JTextArea auxArea ; // to post text of information  12.13.2015
   public JList rulelist ; // to select rule

   protected Tree tree ;   // The Tree this TreePane draws
   public int[] X ;        // where nodes of the tree locate
   public int[] Y ;
   public int selected ;    // index of selected node/fact
   /////
   protected double scale = 1.0 ;  // default
   protected JPopupMenu popedit ; 
   protected TreePane self = this ; 
   //public static DataWindow dw = new DataWindow() ;  
   
   private int [] treewidth ; // holds calculations bottom up

   public JScrollPane scrollPane ;  // access to scroll for drag here 
                                    // defined in sam/gui/Colog.java
   private int mousex, mousey ; 

   // geometric constants based on monospaced ~14pt
   static Font font = 
      new Font((System.getProperty("os.name").indexOf("Mac")>=0 ? "monospaced" : "monospaced"), Font.PLAIN, 14) ; 
   static int nodeHeight = 10 ; 
   static int charWidth = 8 ;   
   static int vGap = 3*nodeHeight ;  // vertical gap between nodes
   static int hGap = 20 ;   // horizontal gap between nodes
   static int hoff = 10 , voff = 20 ; // offsets of tree

    
   /**
     *  Construct TreePane. Use setCologTree(GeologTree) to initialize.
     */
   public TreePane() { 
      this.rule = -1 ; 
      this.setBackground(Color.white) ; 
      this.createToolTip() ;
      this.setToolTipText("<html><font size=\"+0\"><b>COLOG TREE PANE</b><br> &nbsp;Click on tree node to view inference in theory pane.&nbsp;&nbsp;   <br>&nbsp;(<i>and</i> 2nd click) <<br> &nbsp;Drag on background to move tree.&nbsp;&nbsp;     ") ;

      this.addMouseListener( new MouseAdapter() { 
         public void mouseReleased(MouseEvent mm) {  
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR)) ; 
         }
         public void mousePressed(MouseEvent mm) {  
            rule = -1 ; 
            setCursor(new Cursor(Cursor.MOVE_CURSOR)) ; 
            mousex = mm.getX() ; mousey = mm.getY() ; // mark for drag
            double xx = mm.getX()/scale , 
                   yy = mm.getY()/scale ; 
            int select = getSelectedNode((int)xx,(int)yy) ; 
            if (select != -1) { 
               // debug //////////////////////////////////////
               //System.out.println(tree.fact[select].focus) ; 
               ////////////////////////////////////////////////
               selected = select ; 
               self.repaint() ; 
               if (tree.rule[selected] != 0) {
                  String r = "@"+selected+",rule "+tree.rule[selected]+": "+ tree.inference[selected].toString(tree.theory) ; 
                  info.setText(r) ;
                  auxArea.setText(r) ;
                  rulelist.setSelectedIndex(tree.rule[selected]-1) ;
               }
               else {  
                  rulelist.clearSelection() ; rulelist.repaint() ;
                  boolean cmleaf = (tree.counterModels == null ? false : tree.counterModels.contains(selected)) ;
                  if(cmleaf) {
                     info.setText(" Branch Model ") ; 
                     auxArea.setText(" Branch Model ") ; 
                  }
                  else { 
                     info.setText(" ") ; 
                     auxArea.setText(" ") ;
                  }
               }
            }
            else { 
               rulelist.clearSelection() ; rulelist.repaint() ;
               selected = -1 ; 
               rule = -1 ;
               info.setText("  ") ; 
               repaint() ; 
            }
         }
      }) ; 
      // drag scroll the tree
      this.addMouseMotionListener( new MouseMotionAdapter() { 
         public void mouseDragged(MouseEvent mm) { 
            setCursor(new Cursor(Cursor.MOVE_CURSOR)) ; 
            int x = mm.getX(), y = mm.getY(), 
            dx = x - mousex, dy = y - mousey, 
            mousex = x, mousey = y ;        
            JScrollBar hor = scrollPane.getHorizontalScrollBar(), 
                       ver = scrollPane.getVerticalScrollBar() ;
            hor.setValue(hor.getValue() - dx/2) ;
            ver.setValue(ver.getValue() - dy/2) ;
        
         }
      }) ;

   } // end constructor  TreePane() 

   /**
     *  Set the inference Tree this pane displays.
     */ 
   public void setCologTree(Tree t) { 
      this.tree = t ;
      this.X = new int[tree.capacity] ;
      this.Y = new int[tree.capacity] ; 
      this.treewidth = new int[tree.capacity] ; 
      this.selected = -1 ; // none selected
      // fact, parent, inference, focus are in tree !!!
   }

   /**
     * Resize per user 
     */
   public void actionPerformed(ActionEvent a) { 
      if (a.getActionCommand().equals("smaller")) 
          {this.scale = 0.9*scale ; this.repaint() ; }
      else if (a.getActionCommand().equals("fraction"))
         { this.scale = 0.5 ; this.repaint() ; }
      else if (a.getActionCommand().equals("larger"))
         { this.scale = 1.1*scale ; this.repaint() ; }
      else if (a.getActionCommand().equals("fit"))
         { this.fitTree() ; this.repaint() ; }
      else if (a.getActionCommand().equals("original"))
         { this.scale = 1.0 ; this.repaint() ; } 
      else if (a.getActionCommand().equals("zoom"))
         zoom(this.selected) ;
          
   }

   /**
     *  Zoom to view node n.  3 Sep 2014
      */
   protected void zoom(int n) {
      if (n == -1) return ; // no selected fact
      // otherwise
      //this.scale = 1.0 ;
      //this.repaint() ;
      JScrollBar hor = scrollPane.getHorizontalScrollBar(), 
                 ver = scrollPane.getVerticalScrollBar() ; 
      double x = X[n],
             y = Y[n],
             width = this.getPreferredSize().width,  
             height = this.getPreferredSize().height,
             horMax = hor.getMaximum(),
             verMax = ver.getMaximum(),  
             h = x/width,
             v = y/height ;
      hor.setValue((int)(h*horMax-this.getVisibleRect().width/2)) ;  // 
      ver.setValue((int)(v*verMax-50)) ;
   }  // ok for now

   /**
     * If cursor is at x,y (already corrected for scale), 
     * which node is selected ?
     * @return selected node at x,y, -1 if no node at x,y. 
     */
   private int getSelectedNode(int x, int y) { 
      if (tree.activeTheory == null) return -1 ;
      for (int i = 0 ; i <= this.tree.count ; i++) 
         if (X[i]-10 < x && x < X[i]+width(i)+10 && 
              Y[i]-10 < y && y < Y[i]+nodeHeight+10 )
            return i ; 
      return -1 ;
   } 

   //////////////////////////////////////////////////
   
   public double getScale() { 
      return this.scale ; 
   }

   public void setScale(double d) { 
      this.scale = d ; 
   }

   public void setSelected(int i) {
      this.selected = i ; 
      //dw.setText("  ") ; 
      //dw.setTitle("  " ) ; 

   }

   public int getSelected() {
      return this.selected ;  
   }

   //////////////////////////////////////////////////
   // DISPLAY METHODS 
   /**
     *  Draw the inference Tree, focus and any selection.
     */
   public void paintComponent(Graphics g) {
      if (this.tree.activeTheory == null) return ;  // no theory, no paint
      super.paintComponent(g) ; 
      //this.setXY() ;  // RESHAPE now called via update in Colog
      Graphics2D g2 = (Graphics2D)g ;
      g2.scale(scale,scale) ;
      // paint the facts 
      this.infcount = 0 ; 
      for(int i=0 ; i <= this.tree.count ; i++) 
         drawNode(g2, i) ; 
      // paint the links
      for(int i=1 ; i <= this.tree.count ; i++) // 0 has NO parent
         drawLink(g2,this.tree.parent[i],i) ;
      // box selected
      if (this.selected != -1) {   // dash around it ... BLACK
         Polygon P = new Polygon() ; 
         P.addPoint(X[selected]-5,Y[selected]-5) ; 
         int w = width(selected),
             h = nodeHeight ; 
         P.addPoint(X[selected]+w+5,Y[selected]-5) ; 
         P.addPoint(X[selected]+w+5,Y[selected]+h+5) ; 
         P.addPoint(X[selected]-5,Y[selected]+h+5) ; 
         Stroke saveStroke = g2.getStroke() ; 
         float[] dashes = {5.0f,3.0f} ; 
         g2.setStroke(new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_BEVEL,1.0f,dashes,0.0f)) ; 
         g.setColor(Color.black) ; 
         g2.drawPolygon(P) ; 
         g2.setStroke(saveStroke) ; 
      }
      // Show focus as RED dash around -- NOT FOR COLOG
      int f = -1 ;
      if (f != -1) {   // dash around it ... RED
         Polygon P = new Polygon() ; 
         P.addPoint(X[f]-5,Y[f]-5) ; 
         int w = width(f),
             h = nodeHeight ; 
         P.addPoint(X[f]+w+5,Y[f]-5) ; 
         P.addPoint(X[f]+w+5,Y[f]+h+5) ; 
         P.addPoint(X[f]-5,Y[f]+h+5) ; 
         Stroke saveStroke = g2.getStroke() ; 
         float[] dashes = {5.0f,3.0f} ; 
         g2.setStroke(new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_BEVEL,1.0f,dashes,0.0f)) ; 
         g2.setColor(Color.RED) ; 
         g2.drawPolygon(P) ; 
         g2.setStroke(saveStroke) ; 
         g2.setColor(Color.BLACK) ; 
      }
      if (rule == 0)
         info.setText("Click in tree pane before run.") ; 
      else if (rule > 0) 
         info.setText("rule "+rule+": "+infcount+ " inferences") ; 
   } // end paintComponent

   /**
     * Compute width of ith node in tree
     * The label for the node is "fact[i] i" .
     */
   protected int width(int i) { 
      return this.charWidth * (this.tree.fact[i].toString(tree.theory).length() + 1 + Integer.toString(i).length()) ; 
   }

   /**
     * Compute n's children in the tree.
     */
   protected Vector getChildren(int n) {
      Vector<Integer> children = new Vector<Integer>() ; 
      for (int c=0 ; c <= this.tree.count ; c++)
         if (this.tree.parent[c]==n) children.add(c) ; 
      return children ; 
   }

   /**
     * Approximate the width of the tree having root at the ith node. 
     * This is used by setXY to layout the tree.  Used to compute values
     * of treewidth[] bottom-up.
     */
   private int treeWidth(int i) { 
      int width = width(i) + hGap ; // the i-th node itself
      Vector children = this.getChildren(i) ; 
      if (children.isEmpty()) return width ; 
      else { 
         int wc = 0 ; 
         Iterator it = children.iterator() ; 
         while (it.hasNext()) 
            // children already computed
            wc += treewidth[(Integer)(it.next())] ; 
         return Math.max(width,wc) ; 
      }
   }

   /**
     * Approximate the height of the tree.
     */
   public int treeHeight() { 
      int h = 0 ; 
      if (this.tree.count > 0) {       
         for(int i = 0 ; i <= this.tree.count ; i++) {  
            int yy = Y[i] + nodeHeight ; 
            if (yy > h) h = yy ; 
         }
      }
      int hh = (h + 75);  
      return hh ; 
   }

   /**
     *  Set the x,y coordinates of the i-th node
     *  using upper-left corner x,y (for box
     *  containing the subtree).  This recursive method
     *  reshapes the tree.
     */
   private void setXY(int i, int x, int y) { 
      X[i] = x + (treewidth[i] - width(i))/2 ; 
      Y[i] = y ;
      Vector children = this.getChildren(i) ; 
      if (children.size() == 1) { 
         int c = (Integer)children.elementAt(0) ;  // the lone child
         // single child centered under parent
         // X[c] = X[i] + (width(i)-width(c))/2 
         //      = xc +(treewidth[c] -width(c))/2 so 
         int xc = X[i] + (width(i) - treewidth[c])/2 ;
         setXY(c,xc,y + vGap + nodeHeight) ;
      }
      else if (children.size() > 1) {
         int dx = x ; // start at the left
         Iterator it = children.iterator() ; 
         while (it.hasNext()) { 
            int child = ((Integer)(it.next())).intValue() ;
            int wc = treeWidth(child) ; 
            setXY(child,dx,y + vGap + nodeHeight) ;
            dx += wc ;           
         }
      }
   }

   /**
     *  Layout the entire tree.
     */
   public void setXY() { 
      if (tree.activeTheory == null) return ; 
      // pre-compute treewidth bottom-up
      for(int i = tree.count ; i >= 0 ; i--) 
         treewidth[i]=treeWidth(i) ; 
      // now setXY can use treewidth(s)
      setXY(0,hoff,voff) ; 
   }

   /**
     * Prefer this panel to be big enough to 
     * show all of the tree.  
     */
   public Dimension getPreferredSize() {  
      int w = 0 ; int h = 0 ; 
      if (this.tree.count > 0) {       //  ????????
         for(int i = 0 ; i <= this.tree.count ; i++) {  
            int xx = X[i] + width(i) ; 
            int yy = Y[i] + nodeHeight ; 
            if (xx > w) w = xx ; 
            if (yy > h) h = yy ; 
         }
      }
      double ww = (w+25)*scale ;
      double hh = (h+25)*scale + 50 ;  // + ???
      return new Dimension((int)ww,(int)hh) ; 
   }

   /**
     *  Fit tree to panel viewport.
     *  Sets scale. 
     */
   public void fitTree() { 
      int view_width = this.getVisibleRect().width, 
          view_height = this.getVisibleRect().height, 
          tree_width = this.treeWidth(0),
          tree_height = this.treeHeight() ;
      double sw = (double)view_width / (double)tree_width ,
             sh = (double)view_height / (double)tree_height ;
      double s  ; 
      if (sw < 1.0 && sh >= 1.0) s = sw ; 
      else if (sw >= 1.0 && sh < 1.0) s = sh ; 
      else if (sw < 1 && sh < 1) s = Math.min(sw,sh) ; 
      else s = 1.0 ; 
      this.scale = s ; 
      this.repaint() ; 
   }

   private void drawNode(Graphics2D g, int i) { 
      if (this.tree.inference[i] != null) g.setColor(Color.blue) ; 
      else g.setColor(Color.black) ;
      //String branches = (tree.choicesRemaining(i) > 0 ? "|" : "") ; 
      g.setFont(this.font) ;  
      boolean cmleaf = (tree.counterModels == null ? false : tree.counterModels.contains(i)) ;
      if (cmleaf) {  // red fill 
         Polygon P = new Polygon() ; 
         P.addPoint(X[i]-5,Y[i]-5) ; 
         int w = width(i),
             h = nodeHeight ; 
         P.addPoint(X[i]+w+5,Y[i]-5) ; 
         P.addPoint(X[i]+w+5,Y[i]+h+5) ; 
         P.addPoint(X[i]-5,Y[i]+h+5) ; 
         g.setColor(Color.GREEN) ; 
         g.fillPolygon(P) ; 
         g.setColor(Color.BLACK) ; 
      }
      if (tree.rule[i] == rule) {  // red box 
         infcount++ ;
         Polygon P = new Polygon() ; 
         P.addPoint(X[i]-5,Y[i]-5) ; 
         int w = width(i),
             h = nodeHeight ; 
         P.addPoint(X[i]+w+5,Y[i]-5) ; 
         P.addPoint(X[i]+w+5,Y[i]+h+5) ; 
         P.addPoint(X[i]-5,Y[i]+h+5) ; 
         g.setColor(Color.RED) ; 
         g.drawPolygon(P) ; 
         g.setColor(Color.BLACK) ; 
      }
      g.drawString(this.tree.fact[i].toString(tree.theory) + " "+String.valueOf(i)+branches(i), X[i], Y[i]+ nodeHeight) ;
   }
   private String branches(int n) { 
      int r = (tree.open == null || tree.open.get(n) == null ? 0 : tree.open.get(n)) ;
      StringBuffer bf = new StringBuffer() ; // bf.append(" ") ; 
      for(int i = 0 ; i < r ; i++) 
         bf.append(" \\") ;
      return bf.toString() ; 
   }

   private void drawLink(Graphics2D g, int from, int to) { 
      // centers 
      int xf = X[from] + width(from)/2 ,
          yf = Y[from] + (nodeHeight+4)/2 ,  
          xt = X[to] + width(to)/2 ,
          yt = Y[to] + (nodeHeight+4)/2 ; 
      Point fp = getAnchor(from,xt,yt) ,
            tp = getAnchor(to,xf,yf) ;
      g.draw(new Line2D.Float(fp.x,fp.y,tp.x,tp.y)) ;
   }

   /** 
     *  @return the "anchor" Point on the side of ith node 
     *   lying on line from center of ith node to (a,b).  
     */
   public Point getAnchor(int i, int a,int b) {
      int anchorx, anchory ;  // TBD
      int w = width(i) ;
      int h = nodeHeight + 4 ;  // fudge 12/27/2006
      int xc = X[i] + w/2 ;  // center of node xc,yc
      int yc = Y[i] + h/2 ; 
      int d = 2 ; 
      if (a == xc) 
         if (b <= yc) { 
            anchorx = xc ; 
            anchory = Y[i] - d ; 
         }
         else { 
            anchorx = xc ; 
            anchory = Y[i] + h - d ; 
         }
      else if (a < xc) // (a,b) left of node center
         if (b == yc) { 
            anchorx = X[i] - d ;
            anchory = yc ;
         }
         else if (b < yc)   // 1
            if ((yc-b)*w <= (xc-a)*h) { // below diagonal
               anchorx = X[i] - d ; 
               double f = (double)w/(double)(2*(xc-a)) ; 
               anchory = yc - (int)(f*(yc-b)) ; 
            }
            else { // above diagonal 
               double f = (double)h/(double)(2*(yc-b)) ;
               anchorx = xc - (int)(f*(xc-a)) ;
               anchory = Y[i] - d; 
            }
         else   // 2
            if ((b-yc)*w <= (xc-a)*h) { // above diagonal
               anchorx = X[i] - d; 
               double f = (double)w/(double)(2*(xc-a)) ; 
               anchory = yc + (int)(f*(b-yc)) ; 
            }
            else { // above diagonal 
               double f = (double)h/(double)(2*(b-yc)) ;
               anchorx = xc - (int)(f*(xc-a)) ;
               anchory = Y[i] + h + d ; 
            }
      else   // right of node center  a > xc  
         if (b == yc) { 
            anchorx = X[i] + w + d ;
            anchory = yc ;
         }
         else if (b < yc)   // 3 
            if ((yc-b)*w <= (a-xc)*h) { // below diagonal
               anchorx = X[i] + w + d ; 
               double f = (double)w/(double)(2*(a-xc)) ; 
               anchory = yc - (int)(f*(yc-b)) ; 
            }
            else { // above diagonal 
               double f = (double)h/(double)(2*(yc-b)) ;
               anchorx = xc + (int)(f*(a-xc)) ;
               anchory = Y[i] - d ; 
            }
         else   // 4 
            if ((b-yc)*w <= (a-xc)*h) { // above diagonal
               anchorx = X[i] + w + d ; 
               double f = (double)w/(double)(2*(a-xc)) ; 
               anchory = yc + (int)(f*(b-yc)) ; 
            }
            else { // above diagonal 
               double f = (double)h/(double)(2*(b-yc)) ;
               anchorx = xc + (int)(f*(a-xc)) ;
               anchory = Y[i] + h + d ; 
            }
      return new Point(anchorx,anchory) ;
   } // end getAnchor

}



