
package sam.infer ; 

import java.util.* ; 
import sam.lang.* ; 
import sam.util.* ; 


/**
  *    @version 25 Jun 2014
  *    @author &copy; John Fisher
  *    implements factual|subfactual antecedent satisfaction
   */
public class ActiveTabDCRule extends ActiveDCRule { 
  
   /**
     * Construct an ActiveTabDCRule 
      */
   public ActiveTabDCRule(Rule r,Tree dftree,ActiveTheory active,int indx) { 
      super(r,dftree,active,indx) ; 
   }  

   /**
     *  Search current ranges for new antecedent matches.
     *  Uses tree.history to test match for factual factors.
      */
   protected boolean hasMore() { 
      if (factor == A) {  
         factor = A-1 ;    // last actual factor
         boolean bt = backtrack() ; 
         if (!bt) return false ;
      }
      while (factor != -1) {   // until fails to have next
         if ( factor == A ) {  // all factors satisfied  
            if (!forwardCheck(A-1) && !tooComplex(tree.complexity_limit)) { ///NOTE BENE!!!!!
               return true ;  // new state 
            }
            else { // subsumed so backtrack 
               factor = A-1 ;    // last actual factor 
               boolean bt = backtrack() ; 
               if (!bt) return false ;
            }
         }
         else { 
            Functor f = FACTOR[factor] ;
            boolean matched = false ; 
            if (f.factual) matched = factual(factor,f) ; // DOES NOT CHECK CHOICE RANGE
            else if (f.subfactual) { // bucket probe
               if (bucket[factor] == null) {  // initialize bucket
                  // top[factor] <=  index <= bottom[factor]
                  bucket[factor] = tree.table.get(keyHashCode(f)) ; 
                  // move across bucket to get in DC range
                  while (bucket[factor] != null && bucket[factor].index < top[factor]) 
                     bucket[factor] = bucket[factor].next ; 
               } 
               // travel across bucket to attempt match
               while(bucket[factor] != null && bucket[factor].index <= bottom[factor]) {
                  if (f.index == tree.branch[bucket[factor].index].index   // same predicate
                         &&  match_order_args(factor,f,tree.branch[bucket[factor].index])) {  
                     matched = true ;
                     choice[factor] = bucket[factor].index ; // where matched in the tree  
                     break ;
                  }
                  else {  
                     // undo any partial bindings
                     // nullify factor column 
                     for(int v = 0 ; v < U ; v++) 
                        table[v][factor] = null ;     
                     ///// -> bucket
                     bucket[factor] = bucket[factor].next ;   
                  }
               }  // end while
            } // end bucket probe
            else { // linear probe
               int p = choice[factor] ;   
               // travel down tree branch to attempt match
               // avoid calling match for different functors
               while (p <= bottom[factor]) {   // stay in bounds  
                     if (match_order_args(factor,f,tree.branch[p])) {  
                                                   // match arguments? 
                        matched = true ;
                        choice[factor] = p ; // where matched in the tree  
                        break ;
                     }
                     else {  
                        // undo any partial bindings
                        // nullify factor column 
                        for(int v = 0 ; v < U ; v++) 
                           table[v][factor] = null ;     
                        ///// down tree
                        p = activeTheory.jumpto[p] ;   
                     }
               }  // end while
            } // end linear probe
            // test result of attempt(s) to match
            if (matched  && !forwardCheck(factor)) factor++ ; // next factor   
            else { 
               boolean bt = backtrack() ; 
               if (!bt) return false ;
            }
         }
      } 
      return false ; // nothing worked this attempt
   }

   private boolean backtrack() {
      boolean bt = true ; 
      while (bt) {  
         // slide left past factuals 
         if (FACTOR[factor].factual) {  // factual factor
            while (factor != -1 && FACTOR[factor].factual)    
               factor-- ; 
         }
         if (factor == -1) return false ; // bail
         // factor is non-factual
         // clear last bind for factor
         for(int u = 0 ; u < U ; u++) 
            table[u][factor] = null ;  
         if (FACTOR[factor].subfactual) {  // subfactual factor
            if (bucket[factor] != null)  bucket[factor] = bucket[factor].next ;   
            if (bucket[factor] == null) {  // move <- on factor
               factor-- ;  
               if (factor == -1) return false ;  
            }  
            else { 
               if (bucket[factor].index > bottom[factor]) {    // beyond DC range
                  bucket[factor] = null ;  
                  factor-- ;
                  if (factor == -1) return false ;  // and more bt
               }
               else bt = false  ;   
            }
         }
         else { // linear factor
            if (choice[factor] > bottom[factor]) { 
               choice[factor] = top[factor] ; 
               factor-- ;
               if (factor == -1) return false ;  // bail
            } 
            else {
               choice[factor] = activeTheory.jumpto[choice[factor]] ; 
               if (choice[factor] <= bottom[factor]) bt = false ;        
               else { // more bt 
                  choice[factor] = top[factor] ; // back up
                  factor-- ; // left
                  if (factor == -1) return false ;  // bail
               }
            }
         } // linear factor
      } // end while bt
      return true ; // bactrack successful
   }

   /**
     *  Check forwarded consequents after antecedent factor k. 
     *  @return whether any forwarded consequent is already true after
     *  antecedent factor k
      */
   protected boolean forwardCheck(int k) { 
      Index L = this.forward[k] ;
      if (L.isEmpty()) return false ;  // nothing to check 
      boolean flag = false ; 
      Iterator it = L.iterator() ;
      while (it.hasNext()) { 
         Conjunction con = cologRule.consequent.elementAt((Integer)it.next()) ;   
         if (con.factual) flag = tabled(k,con) ;
         else flag = subsumed(con) ;
         if (flag) return true ;  // as soon as possible ;
      }
      return false ; 
   }

   /**
     *  @return whether existential conjunction is true on branch 
     *  after antecedent factor k
      */
   protected boolean subsumed(Conjunction con) { 
      // con is existential conjunction 
      //    but some factors may be factual
      // reset table for existential binding
      for (int v = U ; v < V ; v++)     // [U,V)
         for (int c = A ; c < C ; c++)  // [A,C)
            table[v][c] = null ;  
      // reset consequent portion of choice[] to first
      for (int j = 0 ; j < con.size() ; j++) {              
         Functor f = (Functor)con.elementAt(j) ;           
         choice[A+j] = activeTheory.first[f.index] ;    
      }
      if (satisfied(con)) {  
         tree.subsumed++ ;
         return true ;
      }
      else return false ;
   }

   /**
     *  @return whether factual conjunction is true on branch 
     *  after antecedent factor k.
     *  Note: A+k is "signal" that choice[-] is not to be set.
      */
   protected boolean tabled(int k, Conjunction con) { 
      ///// All con factors are factual 
      Iterator it = con.iterator() ; 
      while (it.hasNext()) { 
         if (!factual(A+k,(Functor)it.next())) return false ; // any fails
      } 
      tree.subsumed++ ;
      return true ;  // none failed
   }

}

/*
hasMore seems ready for Tab

backtrack needs tab mod for subfactual

ActiveDCRule needs amends for bucket[-]:
   When a bucket is restarted it should be run over to 
   choice[factor]  somehow
*/
