package sam.lang ; 

import java.io.* ; 

/**
  *  Bound term, no variables. 
  *  @version 23 May 2014
  *  @author &copy; John Fisher 
   */
public class Fact extends Term { 

   public Fact[] args ;  // stays null for constants ;-)

   /**
     *  A Fact is either a grounded predicate or operator.
      */
   public Fact(int index, int arity) {   
      this.index = index ;     
      if (arity > 0) 
         this.args = new Fact[arity] ;  // set by expand() in ActiveRule
      
   }

   /**
     *  Create string representation. 
     *  @param theory has toString(Theory) info
     */
   public String toString(Theory theory) { 
      if (this.index >= theory.name.length) // SKOLEM CONSTANTS
         return "sk"+(index-theory.name.length) ;
      StringBuffer sb = new StringBuffer() ; 
      // sb.append("#") ; 
      if (theory.type[this.index] == Functor.PRED)   // - op-  
         sb.append(this.args[0].toString(theory) + theory.name[this.index] + this.args[1].toString(theory)  ) ; // NO parens 
      else if (theory.type[this.index] == Functor.BOP)  // (- op - ) 
         sb.append("(" + this.args[0].toString(theory) + theory.name[this.index] + this.args[1].toString(theory) + ")" ) ; 
      else if (theory.type[this.index] == Functor.UOP) 
         sb.append(theory.name[this.index] + this.args[0].toString(theory) ) ; 
      else { 
         sb.append(theory.name[this.index]) ; 
         if (this.args != null) {  
            sb.append("(") ; 
            for(int i=0 ; i < this.args.length ; i++) { 
               sb.append(this.args[i].toString(theory)) ; 
               if(i < this.args.length-1) 
               sb.append(",") ; 
            }
            sb.append(")") ; 

         }
      }
      return sb.toString() ;  
   }

///////////////////////////////////////////////
// Previous method using special instance field
///////////////////////////////////////////////
   /**
     *  Normal call to pre-computed hash value.
     *  @return Fact.hashcode(this)
      */
   public int hashCode() { return Fact.hashcode(this) ; }  
   
   /**
     *  Equality of Facts.
     *  Note: Facts have no variables.
      */
   public boolean equals(Object obj) { 
      if  (!(obj instanceof Fact) ||
              this.index != ((Fact)obj).index || 
                this.args.length != ((Fact)obj).args.length ) return false ;
      // otherwise 
      for(int i = 0 ; i < this.args.length ; i++) 
            if (!this.args[i].equals(((Fact)obj).args[i])) return false ; 
      return true ; 
   }
   
   /**
     *  Make a fact from a ground functor.
      */
   public static Fact toFact(Functor f) { // top-down
      Fact fact = new Fact(f.index,f.arity) ;
      for(int i = 0 ; i < f.args.length ; i++)  
         fact.args[i] = toFact((Functor)f.args[i]) ; 
      return fact ; 
   }

   private static int[] shift = {3,5,7,11,13} ;
 
 
   /**
    * Static hashcode computation for factual hash code 
    *  Note that 31*x = (x << 5) - x, but no "-x" here.
    */
   public static int hashcode(Fact f) { 
      int h = f.index ;
      if (f.args  == null) return h ; 
      for (int i = 0 ; i < f.args.length ; i++) 
         h = (h<<5) + hashcode(f.args[i]) ;
      return h ;
   }     
// h = (h<< f.args[i].index ) + hashcode(f.args[i]) ; 
// somewhat more dispersal but more computation

   /**
     *  @return hashcode using key pattern. 
      */
   public static int hashcode(Fact f, boolean[] key) { 
      int h =f.index ;
      if (f.args  == null) return h ; 
      for (int i = 0 ; i < f.args.length ; i++) 
         h = (h<<5) + (key[i] ? hashcode(f.args[i]) :  0  ) ;
      return h ;
   }

   public static void main(String[] args) { 
      String thy = args[0].trim() ; 
      Theory theory = null ;
      try { 
         sam.lang.antlr.cologLoader loader = 
            new sam.lang.antlr.cologLoader(new FileInputStream(new File(thy))) ; 
         theory = loader.theory ;
      } catch(Exception xx) { System.out.println(xx) ; }
      CologAnalyzer.analyze(theory) ;
      CologAnalyzer.report(theory) ;  
      Rule r = theory.elementAt(0) ; 
      System.out.println(r) ; 
      // r ~  true => term(t1), term(t2).  
      Functor t1 = (Functor)r.consequent.elementAt(0).elementAt(0).args[0] ; 
      Functor t2 = (Functor)r.consequent.elementAt(0).elementAt(1).args[0] ; 
      System.out.println(t1) ; 
      System.out.println(t2) ; 
      Fact f1 = toFact(t1) ;
      Fact f2 = toFact(t2) ;
      System.out.println(f1.toString(theory)+" hashcode(f1)="+hashcode(f1)) ; 
      System.out.println(f2.toString(theory)+" hashcode(f2)="+hashcode(f2)) ; 
      System.out.println(Integer.MAX_VALUE) ; 
   }  
      

}


