package sam.lang ; 

import java.util.* ; 
import sam.util.* ; 
import sam.infer.* ;
import java.io.* ; 

/**
  *  Colog Theory is list of Rules.
  *  @version 2 April 2014 
  *  @author &copy; John Fisher 
  */
public class Theory extends Vector<Rule> { 

   public static int TRUE_INDEX = 0 ;  // name[0] == "true", etc.
   public static int GOAL_INDEX = 1 ;
   public static int FALSE_INDEX = 2 ;
   // set via CologAnalyzer in ActiveTheory
   public String[] name ;                     // print names of functors by index
   public byte[] type ;                       // types of functors by index
   public HashMap<String,String> hints ;      // attribute/value pairs, no prior semantics
   public Vector<Pattern>[] keyPatterns ;     // by index
   protected int count ;                      // used by CologAnalyzer to index 
   public boolean subfactual ;                // need sftable, in sam.infer.Tree,setTheory(-),
                                              // set by CologAnalyzer
   public Theory() {
      super() ;
      hints = new HashMap<String,String>() ;
   } 

   /**
     * A theory is flat if it contains no splitting rules.
      */
   public boolean isFlat() { 
      Iterator r = this.iterator() ;
      while (r.hasNext()) 
         if (  ((Rule)r.next()).consequent.size() >1) return false ;
      return true ;
   }

   /**
     *  1-based getRule(r).  No guard for out of range.
     */
   public Rule getRule(int r) { 
      return (Rule)this.elementAt(r-1) ; 
   }

   /**
     *  String display of this theory
     */
   public String toString() { 
      StringBuffer sb = new StringBuffer() ; 
      for(int i=1 ; i<=this.size() ; i++) {
         sb.append("/*"+i+"*/ ") ; // rule number 1 ...
         sb.append(this.getRule(i).toString()) ;  
         sb.append(".") ;
         if (this.getRule(i).existential>0) sb.append(" /* Existential */") ;
         sb.append("\n") ; 
      }
      return sb.toString() ; 
   }

   /**
     *  HTML display of this theory.
     *  Uses loaded theory attributes for display.
     */
   public String toHTML(int size,Tree tree) { 
      StringBuffer sb = new StringBuffer() ; 
      sb.append("<BLOCKQUOTE>"); 
      Iterator it = this.iterator() ;
      int c = 1 ;
      while (it.hasNext()) {
         Rule rule = (Rule)it.next() ; 
         sb.append(rule.toHTML(tree.activeTheory.qued(c),c,size,rule.U)) ; 
         sb.append("<br>\n") ; 
         c++ ;
      }
      sb.append("<BLOCKQUOTE>"); 
      return sb.toString() ; 
   }

   /**
     *  Display of (substituted) grounded rule,
     *  @param a is And antecedent
     *  @param d is And[] disjunctive consequent
     */
   public String toString(And a, And[] d) { 
      StringBuffer sb = new StringBuffer() ; 
      sb.append(a.toString(this)) ; 
      sb.append(" => ") ; 
      for(int i=0 ; i < d.length ; i++) {
         sb.append(d[i].toString(this)) ; 
         if (i < d.length-1) sb.append(" | ") ; 
      }
      return sb.toString() ; 
   }



   // handy for testing, convert ground Functor to Fact
   private static Fact toFact(Functor fnctr) { 
      Fact f = new Fact(fnctr.index,fnctr.arity) ; 
      for(int i=0 ; i < fnctr.arity ; i++)
         f.args[i] = toFact((Functor)fnctr.args[i]) ; 
      return f ; 
   }

   /**
     *  Test indexing, etc.
     *  
      */
   public static void main(String[] args) { 
   }
   
}

