package sam.lang ; 
 
/**
  * TypedVariable
  * Inference variables (e.g., colog) have no type.
  * @version  2 April 2014
  * @author &copy; John Fisher 
  */
public class TypedVariable extends Variable {

   /**
     * the type or guard for this variable
     */
   public String typename ; 

   /**
     *  Construct a TypedVariable by name and typename.
     */
   public TypedVariable(String n, String tn) { 
      super(n) ; 
      this.typename = tn ; 
   }


   public String getTypeName() {
      return this.typename ;
   }

   /**
     *  @return the (String) name:typename of this Geolog variable.
     */
   public String toFolString() { 
      return this.name +":"+ this.typename ; 
   }
   /**
     *  Clone this TypedVariable.
     */
   public TypedVariable clone() { 
      return new TypedVariable(this.name,this.typename) ; 
   }

   /**
     *  Clone this TypedVariable.
     */
   public Variable unType() { 
      return new Variable(this.name) ; 
   }

   /**
     *  Equality of name and typename.
      */
   public boolean equals(Object obj) { 
      return ((obj instanceof TypedVariable)) && 
               (this.name).equals(((TypedVariable)obj).name) &&
                 (this.typename).equals(((TypedVariable)obj).typename) ;  
   }

   public static void main(String[] args) {
      TypedVariable tv1 = new TypedVariable("crumb","dumb") ;
      TypedVariable tv2 = new TypedVariable("crumb","dumb") ;
      System.out.println(tv1.typename) ;
      System.out.println(tv2.typename) ;
      System.out.println(tv1 instanceof TypedVariable) ;
      System.out.println(tv2 instanceof TypedVariable) ;

      System.out.println(tv1.equals(tv2)) ;
   }

}
