/**
  *  A colog code grammar.
  *  Algebra -- infix functors -- needs to be fully parenthesized.
  *  @author jrfisher@csupomona.edu
  *  30 December 2013 version, hints
  *  BE SURE TO SET GENERATOR PATH: e.g., /colog2.1/sam/lang/antlr
  */
grammar colog ; 

@header {
package sam.lang.antlr ;
import sam.lang.* ; 
import java.util.Vector ; 
}
@lexer::header { package sam.lang.antlr ; }

@members {
/** Store Theory */
Theory cologTheory ;
}

test	: {cologTheory= new Theory() ; }theory   ;   // test() called by cologLoader

theory :	
   hint* ( r=rule  {cologTheory.add($r.value);}  '.')+  ; 

hint : attr=name '=' (n=number { cologTheory.hints.put($attr.value,$n.value) ;} 
                      | v=name { cologTheory.hints.put($attr.value,$v.value) ;} )'.'  ; // attr --> value

rule returns [Rule value]:	
   conjunction '=>' disjunction 
   {$value=new Rule($conjunction.value,$disjunction.value) ; }  ;

disjunction returns  [Disjunction value]:  
   {$value=new Disjunction();} 
   c=conjunction {$value.add($c.value) ; } 
   ((';'| '|') c=conjunction {$value.add($c.value) ; })* ;
 
conjunction returns [Conjunction value] : 
   {$value=new Conjunction(); } 
   p=predicate {$value.add($p.value) ; }  
   (',' p=predicate {$value.add($p.value) ; })* ;

predicate returns [Functor value]
	:	f=functor {$value = $f.value ; } // tricky, user's expression
	(pred t=term  { Vector<Term> v1 = new Vector<Term>() ;  v1.add($value) ; v1.add($t.value) ;    $value = new Functor($pred.text,v1,Functor.PRED) ; }   ) *  // 0 or 1
	 | v=variable pred t=term  {Vector<Term> v2 = new Vector<Term>() ; v2.add($variable.value) ; v2.add($term.value) ; $value = new Functor($pred.text,v2,Functor.PRED) ; }
	 | n=number pred t=term {Vector<Term> v3 = new Vector<Term>() ; v3.add(new Functor($n.value,0,Functor.FCNL)) ; v3.add($t.value) ; $value = new Functor($pred.text,v3,Functor.PRED) ; }
	;

functor  returns [Functor value]  :  
     name '(' terms ')' {$value = new Functor($name.value, $terms.value,Functor.FCNL) ; }           
   | name {$value = new Functor($name.value,0,Functor.FCNL) ; } 
   | '(' t1=term bop t2=term ')' {Vector<Term> v = new Vector<Term>() ;  v.add($t1.value) ; v.add($t2.value) ;  $value = new Functor($bop.text,v,Functor.BOP) ; } 
   | UOP t=term {Vector<Term> v = new Vector<Term>() ;  v.add($t.value) ; $value = new Functor($UOP.text,v,Functor.UOP) ; } 
    ; 
   
terms returns [Vector<Term> value] :  
   {$value = new Vector<Term>() ;} 
   t=term {$value.add($t.value);} 
   (',' t=term {$value.add($t.value);} )* ; 

term returns [Term value] : 
   variable {$value = $variable.value; }
   | number  {$value = new Functor($number.value,0,Functor.FCNL);} 
   | functor {$value = $functor.value ; }   ;

   
variable returns [Variable value] : 
   (UPPER|'_') (LOWER | UPPER | '_' | NUMERIC)*  // like Prolog
   {$value= new Variable($variable.text) ; } ; 

name returns [String value]:	
   '~'* '$'? LOWER (LOWER | UPPER | '_' | NUMERIC)*   {$value = $name.text ;}  
   | QUOTED { $value = $name.text ; } 
   | DQUOTED { $value = $name.text ; } 
   | ALL variable {$value = $name.text ;}
   ; 

number returns [String value]:	// ! GENERATES WARNING !
   NUMERIC+  {$value = $number.text ; }  ;

/* bop's include pred's, but not vice versa */
bop 	:	 
      '$'?( '&' | '|' | ';' | '=>' | '->' | '*' | '#' | '+' | '/' | '\\' | '- ' | '^')  ;  // pred is also bop

pred 	:	 '$'?( '=' | '!=' | '<' | '>' | '>=' | '<=' ) ; 

//neg 	:	  '~' | '~~'| '~~~'  ;  // why need multiple '~' ?

UOP 	:	 '$'? ('-' | '?' | '!') ; 
ALL 	:	 '@' ;   // special universalization multi-operator 


LOWER 	:	 'a'..'z' ; 

UPPER 	:	'A'..'Z' ; 

NUMERIC :	'0'..'9' ; 

WS : (' '|'\t'|'\n'|'\r')+ {skip();} ;

SL_COMMENT : ('%'|'//') ( options {greedy=false;} : . )* ('\n'|'\r') {skip();} ;

ML_COMMENT : '/*' ( options {greedy=false;} : . )* '*/' {skip();}; 

QUOTED	:	'\'' ( options {greedy=false;} : . )* '\'' ;

DQUOTED :	 '\"' ( options {greedy=false;} : .)* '\"' ;    
    
