package sam.lang.antlr ; 

import org.antlr.runtime.* ; 
import sam.lang.* ; 
import java.io.* ; 

/**
  *  Use constructor cologLoader(File) to attempt
  *  load of a colog theory.  Then test for .error. 
  *  The colog theory loaders were created using 
  *  <a href="http://www.antlr.org/">ANTLRWorks</a>
  *  @version 1 January 2013
  *  @author &copy; John Fisher 
   */
public class cologLoader {

   /**
     *  If .error == true, then do not attempt to show result to user.
     *  Otherwise .theory can be shown (but possibly wrong). 
     */
   public boolean error ; 
   public Theory theory ;  

   /**
     *  Load geolog theory via InputStream.  
     */
   public cologLoader(InputStream gl) {
      try { 
         ANTLRInputStream input = 
             new ANTLRInputStream(gl);
         cologLexer lexer = new cologLexer(input);
         CommonTokenStream tokens = new CommonTokenStream(lexer);
         cologParser parser = new cologParser(tokens);
         parser.test();  // try to parse theory
         this.theory = parser.cologTheory ; 
      } 
      catch(Exception xxx) { 
         //System.out.println(xxx) ;  // dialog?
         this.error = true ; 
      }
      // Would the theory print, using toString()'s ???
      try { // will print?
         this.theory.toString() ;
      }  
      catch (Exception yyy) { this.error = true ; }  
  }

   /**
     *  Unit Test for GeologLoader.
     */
   public static void main(String[] args) { 
      FileInputStream in = null ;
      try { in = new FileInputStream(new File(args[0])); } catch(Exception x) { System.out.println(x) ; } 
      cologLoader loader = 
        new cologLoader(in) ; // antlr prints own parse errors
      // NO THEORY
      if (loader.theory == null) System.out.println("NO FILE") ; 
      // BAD THEORY but can be displayed
      else if (loader.error) { // cannot show to user!
         System.out.println("Error") ; 
      }
      // PRINTABLE at least
      else System.out.println(loader.theory) ; 
   }

}

