
package sam.util ; 

/**
  *  Quick and small nonneg int stack.  
  *  Intended to hold bactracking points for 
  *  DF geolog search.
  */
public class S {
   private int[] data ; 
   private int top ; // top of stack 
   private int capacity ; 
   
   public S(int cap) { 
      this.capacity = cap ; 
      this.top = -1 ;   // empty
      this.data = new int[cap] ; 
   }

   /*** Clear this stack. */
   public void clear() { this.top = -1 ; }

   /**
     *  @return top of stack, -1 if empty
     */
   public int peek() { 
      if (top == -1) return -1 ; 
      else return data[top] ;   
   }


   /**
     *  pop the stack
     *  @return top of stack, -1 if empty
     */
   public int pop() { 
      if (top == -1) return -1 ; 
      else { 
         top-- ;
         return data[top+1] ; 
      }  
   }

   public void push(int n) { 
      if (top == capacity-1) { // expand because full
         int newcap = 2*capacity ;
         int[] newdata = new int[2*capacity] ; 
         for (int i = 0 ; i < capacity ; i++) 
            newdata[i] = data[i] ;
         top++ ;
         data = newdata ;
         capacity = newcap ;
         data[top] = n ;
         
      }
      else { 
         top++ ;
         data[top] = n ;
      }
   }

   public int size() { 
      return top+1 ;
   }
   
   public boolean isEmpty() { 
      return (top == -1) ;
   } 

   public static void main(String[] args) { 
      // test overflow mechanism
      S s = new S(10) ; 
      for (int i = 1 ; i < 1000 ; i++) s.push(i) ; 
      System.out.println(s.size()) ; 

   }

   public void show() { 
      System.out.print("<") ;
      for(int i = 0 ; i <=top ; i++)
         System.out.print(data[i]+" ") ;
      System.out.println(">") ;
   }  


} // end S
